@@ -8,7 +8,7 @@ use warnings;
 ###########################################################################
 
 use Readonly;
-Readonly my %text_strings => (
+Readonly my %TEXT_STRINGS => (
     'MYAPP_HELLO' => q[Light goes on!],
     'MYAPP_GOODBYE' => q[Light goes off!],
     'MYAPP_PROMPT'
@@ -24,7 +24,7 @@ Readonly my %text_strings => (
 { package MyApp::L::Homer; # module
     sub get_text_by_key {
         my (undef, $msg_key) = @_;
-        return $text_strings{$msg_key};
+        return $TEXT_STRINGS{$msg_key};
     }
 } # module MyApp::L::Homer
 
